@@ -6,6 +6,8 @@ void ThrowTypeError(std::string error) {
 }
 }  // namespace Nan
 
+// anonymous namepace for helpers
+namespace {
 struct MongoCryptStatusDeleter {
     void operator()(mongocrypt_status_t* status) {
         mongocrypt_status_destroy(status);
@@ -37,11 +39,7 @@ std::string StringFromBinary(mongocrypt_binary_t* binary) {
 mongocrypt_binary_t* BufferToBinary(v8::Local<v8::Object> node_buffer) {
     uint8_t* buffer = (uint8_t*)node::Buffer::Data(node_buffer);
     size_t buffer_len = node::Buffer::Length(node_buffer);
-
-    uint8_t* buffer_copy = new uint8_t[buffer_len];
-    memcpy(buffer_copy, buffer, buffer_len);
-
-    return mongocrypt_binary_new_from_data(buffer_copy, buffer_len);
+    return mongocrypt_binary_new_from_data(buffer, buffer_len);
 }
 
 v8::Local<v8::Object> BufferFromBinary(mongocrypt_binary_t* binary) {
@@ -121,7 +119,8 @@ std::string errorStringFromStatus(mongocrypt_ctx_t* context) {
     return errorMessage;
 }
 
-Nan::Persistent<v8::Function> MongoCrypt::constructor;
+}  // anonymous namespace
+
 NAN_MODULE_INIT(MongoCrypt::Init) {
     v8::Local<v8::FunctionTemplate> tpl = Nan::New<v8::FunctionTemplate>(New);
     tpl->SetClassName(Nan::New("MongoCrypt").ToLocalChecked());
@@ -138,7 +137,7 @@ NAN_MODULE_INIT(MongoCrypt::Init) {
 
     Nan::SetAccessor(itpl, Nan::New("status").ToLocalChecked(), Status);
 
-    constructor.Reset(Nan::GetFunction(tpl).ToLocalChecked());
+    constructor().Reset(Nan::GetFunction(tpl).ToLocalChecked());
     Nan::Set(
         target, Nan::New("MongoCrypt").ToLocalChecked(), Nan::GetFunction(tpl).ToLocalChecked());
 }
@@ -164,7 +163,7 @@ void MongoCrypt::logHandler(mongocrypt_log_level_t level,
 }
 
 
-void MaybeSetCryptoHookErrorStatus(v8::Local<v8::Value> result, mongocrypt_status_t *status) {
+static void MaybeSetCryptoHookErrorStatus(v8::Local<v8::Value> result, mongocrypt_status_t *status) {
     if (!result->IsObject()) {
         return;
     }
@@ -400,7 +399,9 @@ NAN_METHOD(MongoCrypt::New) {
                     return;
                 }
 
-                if (!mongocrypt_setopt_kms_providers(crypt.get(), BufferToBinary(kmsProvidersOptions))) {
+                std::unique_ptr<mongocrypt_binary_t, MongoCryptBinaryDeleter> kmsProvidersBinary(
+                    BufferToBinary(kmsProvidersOptions));
+                if (!mongocrypt_setopt_kms_providers(crypt.get(), kmsProvidersBinary.get())) {
                     Nan::ThrowTypeError(errorStringFromStatus(crypt.get()));
                     return;
                 }
@@ -416,7 +417,9 @@ NAN_METHOD(MongoCrypt::New) {
                     return;
                 }
 
-                if (!mongocrypt_setopt_schema_map(crypt.get(), BufferToBinary(schemaMapBuffer))) {
+                std::unique_ptr<mongocrypt_binary_t, MongoCryptBinaryDeleter> schemaMapBinary(
+                    BufferToBinary(schemaMapBuffer));
+                if (!mongocrypt_setopt_schema_map(crypt.get(), schemaMapBinary.get())) {
                     Nan::ThrowTypeError(errorStringFromStatus(crypt.get()));
                     return;
                 }
@@ -493,7 +496,7 @@ NAN_METHOD(MongoCrypt::New) {
 
     const int argc = 1;
     v8::Local<v8::Value> argv[argc] = {info[0]};
-    v8::Local<v8::Function> ctor = Nan::New<v8::Function>(MongoCrypt::constructor);
+    v8::Local<v8::Function> ctor = Nan::New<v8::Function>(constructor());
     info.GetReturnValue().Set(Nan::NewInstance(ctor, argc, argv).ToLocalChecked());
 }
 
@@ -671,6 +674,36 @@ NAN_METHOD(MongoCrypt::MakeDataKeyContext) {
         return;
     }
 
+    v8::Local<v8::Object> options = Nan::To<v8::Object>(info[1]).ToLocalChecked();
+    auto ALT_NAMES_KEY = Nan::New("keyAltNames").ToLocalChecked();
+    if (Nan::Has(options, ALT_NAMES_KEY).FromMaybe(false)) {
+        v8::Local<v8::Value> keyAltNames = Nan::Get(options, ALT_NAMES_KEY).ToLocalChecked();
+
+        if (keyAltNames->IsArray()) {
+            v8::Local<v8::Array> keyAltNamesArray = v8::Local<v8::Array>::Cast(keyAltNames);
+            uint32_t keyAltNamesLength = keyAltNamesArray->Length();
+            for (uint32_t i = 0; i < keyAltNamesLength; i += 1) {
+                if (Nan::Has(keyAltNamesArray, i).FromMaybe(false)) {
+                    v8::Local<v8::Object> keyAltName =
+                        Nan::To<v8::Object>(Nan::Get(keyAltNamesArray, i).ToLocalChecked())
+                            .ToLocalChecked();
+                    if (!node::Buffer::HasInstance(keyAltName)) {
+                        // We should never get here
+                        Nan::ThrowTypeError("Serialized keyAltName must be a Buffer");
+                        return;
+                    }
+
+                    std::unique_ptr<mongocrypt_binary_t, MongoCryptBinaryDeleter> binary(
+                        BufferToBinary(keyAltName));
+                    if (!mongocrypt_ctx_setopt_key_alt_name(context.get(), binary.get())) {
+                        Nan::ThrowTypeError(errorStringFromStatus(context.get()));
+                        return;
+                    }
+                }
+            }
+        }
+    }
+
     if (!mongocrypt_ctx_datakey_init(context.get())) {
         Nan::ThrowTypeError(errorStringFromStatus(context.get()));
         return;
@@ -680,7 +713,6 @@ NAN_METHOD(MongoCrypt::MakeDataKeyContext) {
     info.GetReturnValue().Set(result);
 }
 
-Nan::Persistent<v8::Function> MongoCryptContext::constructor;
 NAN_MODULE_INIT(MongoCryptContext::Init) {
     v8::Local<v8::FunctionTemplate> tpl = Nan::New<v8::FunctionTemplate>();
     tpl->SetClassName(Nan::New("MongoCryptContext").ToLocalChecked());
@@ -697,7 +729,7 @@ NAN_MODULE_INIT(MongoCryptContext::Init) {
     Nan::SetAccessor(itpl, Nan::New("status").ToLocalChecked(), Status);
     Nan::SetAccessor(itpl, Nan::New("state").ToLocalChecked(), State);
 
-    constructor.Reset(Nan::GetFunction(tpl).ToLocalChecked());
+    constructor().Reset(Nan::GetFunction(tpl).ToLocalChecked());
     Nan::Set(target,
              Nan::New("MongoCryptContext").ToLocalChecked(),
              Nan::GetFunction(tpl).ToLocalChecked());
@@ -705,7 +737,7 @@ NAN_MODULE_INIT(MongoCryptContext::Init) {
 
 v8::Local<v8::Object> MongoCryptContext::NewInstance(mongocrypt_ctx_t* context) {
     Nan::EscapableHandleScope scope;
-    v8::Local<v8::Function> ctor = Nan::New<v8::Function>(MongoCryptContext::constructor);
+    v8::Local<v8::Function> ctor = Nan::New<v8::Function>(constructor());
     v8::Local<v8::Object> object = Nan::NewInstance(ctor).ToLocalChecked();
     MongoCryptContext* class_instance = new MongoCryptContext(context);
     class_instance->Wrap(object);
@@ -794,7 +826,6 @@ NAN_METHOD(MongoCryptContext::Finalize) {
     info.GetReturnValue().Set(buffer);
 }
 
-Nan::Persistent<v8::Function> MongoCryptKMSRequest::constructor;
 NAN_MODULE_INIT(MongoCryptKMSRequest::Init) {
     v8::Local<v8::FunctionTemplate> tpl = Nan::New<v8::FunctionTemplate>();
     tpl->SetClassName(Nan::New("MongoCryptKMSRequest").ToLocalChecked());
@@ -808,7 +839,7 @@ NAN_MODULE_INIT(MongoCryptKMSRequest::Init) {
     Nan::SetAccessor(itpl, Nan::New("endpoint").ToLocalChecked(), Endpoint);
     Nan::SetAccessor(itpl, Nan::New("message").ToLocalChecked(), Message);
 
-    constructor.Reset(Nan::GetFunction(tpl).ToLocalChecked());
+    constructor().Reset(Nan::GetFunction(tpl).ToLocalChecked());
     Nan::Set(target,
              Nan::New("MongoCryptKMSRequest").ToLocalChecked(),
              Nan::GetFunction(tpl).ToLocalChecked());
@@ -816,7 +847,7 @@ NAN_MODULE_INIT(MongoCryptKMSRequest::Init) {
 
 v8::Local<v8::Object> MongoCryptKMSRequest::NewInstance(mongocrypt_kms_ctx_t* kms_context) {
     Nan::EscapableHandleScope scope;
-    v8::Local<v8::Function> ctor = Nan::New<v8::Function>(MongoCryptKMSRequest::constructor);
+    v8::Local<v8::Function> ctor = Nan::New<v8::Function>(constructor());
     v8::Local<v8::Object> object = Nan::NewInstance(ctor).ToLocalChecked();
     MongoCryptKMSRequest* class_instance = new MongoCryptKMSRequest(kms_context);
     class_instance->Wrap(object);
@@ -878,7 +909,7 @@ NAN_METHOD(MongoCryptKMSRequest::AddResponse) {
     mongocrypt_kms_ctx_feed(mckr->_kms_context, reply_bytes.get());
 }
 
-NAN_MODULE_INIT(Init) {
+static NAN_MODULE_INIT(Init) {
     MongoCrypt::Init(target);
     MongoCryptContext::Init(target);
     MongoCryptKMSRequest::Init(target);
